// @ myth - client - server.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// This file contain rutines related to the server channel.

// @@ include

#include "client/server.h"
#include "common/error.h"

// @@ const

// Time(s) to wait for connection to server.
static const int MaxTries = 30;
static const int SleepTime = 1;

// @@ void CheckServer(CHANNEL* channel, INPUT* input, OUTPUT* output)

void CheckServer(CHANNEL* channel, INPUT* input, OUTPUT* output)
{
    // The following is used to contain information read from the server
    PAGE page;
    STRING string;
    char c;
    coor pos;
    inventory_type bunch;

    for(; /* Until SERVER_READY received */ ;)
    {
	switch(channel->GetChar())
	{	
	case SERVER_IDLE:
	    bug.error("SERVER::read", "SERVER_IDLE received");
	    break;
	case SERVER_NEWS:
	    output->news.insert(channel->GetString());
	    break;
	case SERVER_STATUS:
	    status_type status = channel->GetChar();
	    status_value value = channel->GetLong();
	    output->status.set(status, value);
	    break;
	case SERVER_INVENTORY:
	    bunch = channel->GetChar();
	    page.clear();
	    string = channel->GetString();
	    while(string.length() > 0)
	    {
		page.append(string);
		string = channel->GetString();
	    }
	    output->inventory.create(bunch, &page);
	    break;
	case SERVER_MAP:
	    output->dungeon.clear();
	    output->dungeon.load(channel);
	    break;
	case SERVER_HOME:
	    output->dungeon.home(channel->GetCoor());
	    break;
	case SERVER_PLOT:
	    pos = channel->GetCoor();
	    c = channel->GetChar();
	    output->dungeon.plot(pos, c);
	    break;
	case SERVER_PRINT:
	    pos = channel->GetCoor();
	    string = channel->GetString();
	    output->dungeon.print(pos, string);
	    break;
	case SERVER_EFFECT:
	    pos = channel->GetCoor();
	    c = channel->GetChar();
	    output->dungeon.effect(pos, c);
	    break;
	case SERVER_REQUEST:
	    output->cover.clear();
	    input->ask(channel);
	    break;
	case SERVER_PAGE:
	    string = channel->GetString();
	    while(string.length() > 0)
	    {
		input->PutPage(string);
		string = channel->GetString();
	    }
	    break;
	case SERVER_READY:
	    return;
	default:
	    bug.error("SERVER::read", "Unknown command received");
	    break;
	}
    }
}

// @@ err StepDaemon(int tries)
//
// Called if connection to daemon is delayed.

err StepDaemon(int tries)
{
    switch(tries)
    {
    case 1:
	(void) fflush(stdout);
	(void) fprintf(stderr, "\nWaiting for connection to myth server");
	(void) fflush(stderr);
	bug.warning("StepDaemon", "Waiting for connection");
	return SUCCES;
    default:
	(void) fprintf(stderr, ".");
	(void) fflush(stderr);
	sleep(SleepTime);
	return SUCCES;
    case MaxTries:
	(void) fprintf(stderr, "\nConnection timed out.\n");
	(void) fflush(stderr);
	bug.warning("::StepDaemon", "Connection timed out");
	exit(1);
    }
    return ERROR;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
