// @ myth - common - common.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// Common include files and definitions.
//
// This file should be included everywhere!

// @@ begin

#ifndef COMMON_HEADER
#define COMMON_HEADER

// @@ enum bool

enum bool { FALSE = 0, TRUE = 1 };

// @@ include

overload compare;

#include <sys/types.h>
#include <osfcn.h>
#include <libc.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/socket.h>

#ifdef __GNUG__
#include "common/gnu.h"
#else
#include "common/cfront.h"
#endif

#ifdef fileno
#undef fileno			// I don't use stdio (or streams...)
#endif fileno

// @@ type

// @@@ enum err

enum err { ERROR = -1, SUCCES = 0 };

// @@@ enum cont

enum cont { CONTINUE, DONE };

// @@@ enum force

enum force { FORCE, CHECK };

// @@@ enum confirmation

enum confirmation { NO, YES, QUIT, CONFIRMATION_END,
		    CONFIRMATION_ERROR = ERROR }; 

// @@@ enum comp
//
// Used with strcmp()

enum comp { LESS_THAN = -1, EQUAL = 0, GREATER_THAN = 1};

// @@@ enum direction

// @@@@ enum direction

enum direction
{
    DIRECTION_NOWHERE,
    DIRECTION_NORTH,
    DIRECTION_NORTH_EAST,
    DIRECTION_EAST,
    DIRECTION_SOUTH_EAST,
    DIRECTION_SOUTH,
    DIRECTION_SOUTH_WEST,
    DIRECTION_WEST,
    DIRECTION_NORTH_WEST,
    DIRECTION_UP,
    DIRECTION_DOWN,
    DIRECTION_END,
    DIRECTION_ERROR = ERROR
};

// @@@@ extern

extern const int dx[DIRECTION_END];
extern const int dy[DIRECTION_END];
extern const int dz[DIRECTION_END];

// @@@@ bool ValidDirection(direction dir)

inline bool ValidDirection(direction dir)
{ return dir >= 0 && dir < DIRECTION_END; }

// @@@ struct coor

// @@@@ struct coor

struct coor
{
    short x;
    short y;
    inline void move(direction dir, int length = 1)
    { x += dx[dir] * length; y += dy[dir] * length; }
    inline bool const operator==(const coor& pos) CONST_MEMBER
    { return x == pos.x && y == pos.y; }
    inline bool const operator!=(const coor& pos) CONST_MEMBER
    { return x != pos.x || y != pos.y; }
    inline bool const operator>=(const coor& pos) CONST_MEMBER
    { return x >= pos.x && y >= pos.y; }
    inline bool const operator<=(const coor& pos) CONST_MEMBER
    { return x <= pos.x && y <= pos.y; }
    inline bool const operator<(const coor& pos) CONST_MEMBER
    { return x < pos.x && y < pos.y; }
    inline bool const operator>(const coor& pos) CONST_MEMBER
    { return x > pos.x && y > pos.y; }
    inline coor const operator+(const coor& pos) CONST_MEMBER
    { return coor(x + pos.x, y + pos.y); }
    inline coor const operator-(const coor& pos) CONST_MEMBER
    { return coor(x - pos.x, y - pos.y); }
    inline coor const operator/(int i) CONST_MEMBER
    { return coor(x / i, y / i); }
    inline coor const operator*(int i) CONST_MEMBER
    { return coor(x * i, y * i); }
    inline const coor& operator=(const coor& pos)
    { x = pos.x; y = pos.y; return pos; }
    inline void operator+=(const coor& pos)
    { x += pos.x; y += pos.y; }
    inline void operator-=(const coor& pos)
    { x -= pos.x; y -= pos.y; }
    inline void operator*=(int i)
    { x *= i; y *= i; }
    inline void operator/=(int i)
    { x /= i; y /= i; }
    inline coor(int x, int y)
    { coor::x = x; coor::y = y; }
    inline coor(const coor& pos)
    { x = pos.x; y = pos.y; }
    inline coor()
    { x = y = 0; }
};

// @@@@ const coor NoWhere, Origo

static const coor NoWhere = coor(-32001, -32001);
static const coor Origo = coor(0, 0);

// @@@@ class region

class region : public coor
{
    coor from;
    coor to;
public:
    void NextX();
    void NextY();
    bool check();
    const coor& operator=(const region& reg);
    region();
    region(const region& reg);
    region(const coor& start, const coor& target);
};

// @@@@ class straight

class straight : public coor
{
    coor to;
    coor diff;
    coor step;
    int xcrit;
    int epsilon;
    bool done;
public:
    void next();
    bool check()
    { return !done; }
    straight(const coor& start, const coor& target);
};

// @@@ typedef (*)

// pointer to comp function(void*, void*)
typedef comp (*pfEpVpV)(void*, void*);

// pointer to void function(const char*)
typedef void (*pfVpcC)(const char*);

// pointer to void function(void)
typedef void (*pfVV)();

// pointer to void function(char)
typedef void (*pfVC)(char);

// pointer to char* function(void)
typedef char* (*pfpCV)(void);

// pointer to err function(int)
typedef err (*pfei)(int);

// @@ const

// @@@ buffers

// Maximum length for a news messages etc.
static const int MessageSize = 1024;

// @@@ numbers

enum { PLURAL, SINGULAR };

// @@ function

// @@@ comp compare(int x, int y)

inline comp compare(int x, int y)
{ return (x < y) ? LESS_THAN : (x > y) ? GREATER_THAN : EQUAL; }

// @@@ comp compare(long x, long y)

inline comp compare(long x, long y)
{ return (x < y) ? LESS_THAN : (x > y) ? GREATER_THAN : EQUAL; }

// @@@ bool IsVowel(char c)

inline bool IsVowel(char c)
{ return strchr("aeiouyAEIOUY", c) != NULL; }

// @@@ int log(long l)

extern int log(long l);

// @@@ int sqr(int x)

inline int sqr(int x)
{ return x * x; }

// @@@ int max(int a, int b)

inline int max(int a, int b)
{ return (a > b) ? a : b; }

// @@@ int min(int a, int b)

inline int min(int a, int b)
{ return (a < b) ? a : b; }

// @@@ int abs(int x)

inline int abs(int x)
{ return x < 0 ? -x : x; }

// @@@ coor abs(coor pos)

inline coor abs(coor pos)
{ return coor(abs(pos.x), abs(pos.y)); }

// @@@ int MyLog(long l)

extern int MyLog(long l);

// @@@ bool NextTo(coor a, coor b)

extern bool NextTo(coor a, coor b);

// @@ end

#endif COMMON_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
