// @ myth - server - hiscore.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/hiscore.h"
#include "common/binary.h"
#include "common/page.h"
#include "common/format.h"

// @@ const int MaxScoreEntries

static const int MaxScoreEntries = 100;

// @@ SCORE_ENTRY

// @@@ STRING SCORE_ENTRY::print()

STRING SCORE_ENTRY::print()
{
    return format("%12ld %s %s.", score(), name(), why());
}
// @@@ void SCORE_ENTRY::save(BINARY* binary)

void SCORE_ENTRY::save(BINARY* binary)
{
    binary->PutLong(MyScore);
    binary->PutString(name());
    binary->PutString(why());
}

// @@@ SCORE_ENTRY::SCORE_ENTRY(BINARY* binary)

SCORE_ENTRY::SCORE_ENTRY(BINARY* binary)
{
    MyScore = binary->GetLong();
    name = binary->GetString();
    why = binary->GetString();
}

// @@@ SCORE_ENTRY::SCORE_ENTRY(long s, STRING n, STRING w)

SCORE_ENTRY::SCORE_ENTRY(long s, STRING n, STRING w)
{
    MyScore = s;
    name = n;
    why = w;
}

// @@ HI_SCORE

// @@@ void HI_SCORE::erase()

void HI_SCORE::erase()
{
    delete current();
    list.remove();
}

// @@@ void HI_SCORE::page(PAGE* page)

void HI_SCORE::page(PAGE* page)
{
    reset();

    while(current() != NULL)
    {
	page->append(current()->print());
	next();
    }
}

// @@@ void HI_SCORE::add(SCORE_ENTRY* entry)

void HI_SCORE::add(SCORE_ENTRY* entry)
{
    if(entry->score() <= MinScore)
    {
	delete entry;
	return;
    }
    int count = 0;
    
    reset();
    while(current() != NULL && current()->score() >= entry->score())
	next();
    
    if(count < MaxScoreEntries)
	insert(entry);
    else 
    {
	while(current() != NULL && count < MaxScoreEntries)
	{
	    count++;
	    next();
	}
	if(current() != NULL)
	{
	    MinScore = current()->score();
	    erase();
	}
    }
}

// @@@ void HI_SCORE::load(BINARY* binary)

void HI_SCORE::load(BINARY* binary)
{
    MinScore = binary->GetLong();
    
    while(binary->GetChar() == 1)
	add(new SCORE_ENTRY(binary));
}

// @@@ void HI_SCORE::save(BINARY* binary)

void HI_SCORE::save(BINARY* binary)
{
    binary->PutLong(MinScore);
    
    reset();
    while(current() != NULL)
    {
	binary->PutChar(1);
	current()->save(binary);
	next();
    }
    binary->PutChar(0);
}

// @@@  HI_SCORE::HI_SCORE()

HI_SCORE::HI_SCORE()
{
    MinScore = 0;
}

// @@@  HI_SCORE::~HI_SCORE()

HI_SCORE::~HI_SCORE()
{
    reset();
    {
	while(current() != NULL)
	    erase();
    }
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
