// @ myth - server - ray.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef RAY_HEADER
#define RAY_HEADER

// @@ include

#include "common/common.h"
#include "server/object.h"

struct BUNCH;
struct COUNTER;
struct MONSTER;
struct LEVEL;

// @@ Ray Weapons

// @@@ RAY_WEAPON

class RAY_WEAPON : public OBJECT
{
public:
    int HitBonus(MONSTER*, MONSTER*)
    { return 25; }
};

// @@@ COLD_BLOW

class COLD_BLOW : public RAY_WEAPON
{
    // Identification
    //
public:
    const char* name()
    { return "cold blow"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 75; }
    damage_type DamageType()
    { return DAMAGE_COLD; }
    //
    // Administration
    //
    COLD_BLOW() { }
};

extern COLD_BLOW TheColdBlow;

// @@@ MAGIC_MISSILE

class MAGIC_MISSILE : public RAY_WEAPON
{
    // Identification
    //
public:
    const char* name()
    { return "magic missile"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 15; }
    damage_type DamageType()
    { return DAMAGE_MAGIC; }
    //
    // Administration
    //
    MAGIC_MISSILE() { }
};

extern MAGIC_MISSILE TheMagicMissile;

// @@@ FLAME

class FLAME : public RAY_WEAPON
{
    // Identification
    //
public:
    const char* name()
    { return "flame"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 250; }
    damage_type DamageType()
    { return DAMAGE_FLAME; }
    //
    // Administration
    //
    FLAME() { }
};

extern FLAME TheFlame;

// @@@ LIGHTENING_BOLT

class LIGHTENING_BOLT : public RAY_WEAPON
{
    // Identification
    //
public:
    const char* name()
    { return "lightening bolt"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 40; }
    damage_type DamageType()
    { return DAMAGE_LIGHT; }
    //
    // Administration
    //
    LIGHTENING_BOLT() { }
};

extern LIGHTENING_BOLT TheLighteningBolt;

// @@ RAY

// @@@ class RAY

class RAY
{
protected:
    int power;
    virtual void effect(LEVEL*, MONSTER*, coor);
public:
    cont zap(LEVEL* level, MONSTER* monster, coor pos);
    RAY(int value);
};

// @@@ class RAY_MONSTER

class RAY_MONSTER : public RAY
{
protected:
    virtual void touch(LEVEL*, MONSTER*);
    void effect(LEVEL* level, MONSTER*, coor pos);
public:
    RAY_MONSTER(int value) : (value) { }
};

// @@@ class RAY_FUNGUS

class RAY_FUNGUS : public RAY_MONSTER
{
protected:
    void touch(LEVEL* level, MONSTER* monster);
public:
    RAY_FUNGUS(int value) : (value) { }
};

// @@@ class RAY_SLEEP

class RAY_SLEEP : public RAY_MONSTER
{
protected:
    void touch(LEVEL* level, MONSTER* monster);
public:
    RAY_SLEEP(int value) : (value) { }
};

// @@@ class RAY_LIFE

class RAY_LIFE : public RAY_MONSTER
{
protected:
    void touch(LEVEL* level, MONSTER* monster);
public:
    RAY_LIFE(int value) : (value) { }
};

// @@@ class RAY_DEATH

class RAY_DEATH : public RAY_MONSTER
{
protected:
    void touch(LEVEL* level, MONSTER* monster);
public:
    RAY_DEATH(int value) : (value) { }
};

// @@@ class RAY_ABILITY

class RAY_ABILITY : public RAY_MONSTER
{
protected:
    virtual COUNTER* ability();
    void touch(LEVEL*, MONSTER* monster);
public:
    RAY_ABILITY(int value) : (value) { }
};

// @@@ class RAY_SLOW

class RAY_SLOW : public RAY_ABILITY
{
protected:
    COUNTER* ability();
public:
    RAY_SLOW(int value) : (value) { }
};

// @@@ class RAY_LIT

class RAY_LIT : public RAY_ABILITY
{
protected:
    COUNTER* ability();
public:
    RAY_LIT(int value) : (value) { }
};

// @@@ class RAY_FAST

class RAY_FAST : public RAY_ABILITY
{
protected:
    COUNTER* ability();
public:
    RAY_FAST(int value) : (value) { }
};

// @@@ class RAY_MISSILE

class RAY_MISSILE : public RAY
{
protected:
    virtual OBJECT* weapon();
    void effect(LEVEL* level, MONSTER* monster, coor pos);
public:
    RAY_MISSILE(int value) : (value) { }
};

// @@@ class RAY_COLD

class RAY_COLD : public RAY_MISSILE
{
protected:
    OBJECT* weapon()
    { return &TheColdBlow; }
public:
    RAY_COLD(int value) : (value) { }
};

// @@@ class RAY_MAGIC_MISSILE

class RAY_MAGIC_MISSILE : public RAY_MISSILE
{
protected:
    OBJECT* weapon()
    { return &TheMagicMissile; }
public:
    RAY_MAGIC_MISSILE(int value) : (value) { }
};

// @@@ class RAY_FLAME

class RAY_FLAME : public RAY_MISSILE
{
protected:
    OBJECT* weapon()
    { return &TheFlame; }
public:
    RAY_FLAME(int value) : (value) { }
};

// @@@ class RAY_LIGHTENING_BOLT

class RAY_LIGHTENING_BOLT : public RAY_MISSILE
{
protected:
    OBJECT* weapon()
    { return &TheLighteningBolt; }
public:
    RAY_LIGHTENING_BOLT(int value) : (value) { }
};

// @@@ class RAY_OBJECT

class RAY_OBJECT : public RAY
{
protected:
    virtual cont touch(LEVEL*, coor, MONSTER*, BUNCH*, OBJECT*);
    virtual void TouchMonster(LEVEL* level, coor pos, MONSTER* monster,
			      BUNCH* bunch, MONSTER* subject);
    void effect(LEVEL* level, MONSTER* monster, coor pos);
public:
    RAY_OBJECT(int value) : (value) { }
};

// @@@ class RAY_POLYMORPH

class RAY_POLYMORPH : public RAY_OBJECT
{
protected:
    cont touch(LEVEL* level, coor, MONSTER*, BUNCH* bunch, OBJECT* object);
public:
    RAY_POLYMORPH(int value) : (value) { }
};

// @@@ class RAY_APPORT

class RAY_APPORT : public RAY_OBJECT
{
protected:
    cont touch(LEVEL* level, coor, MONSTER* monster, BUNCH*, OBJECT* object);
    void TouchMonster(LEVEL* level, coor,
		      MONSTER* monster, BUNCH*, MONSTER* subject);
public:
    RAY_APPORT(int value) : (value) { }
};

// @@@ class RAY_TELEPORT

class RAY_TELEPORT : public RAY_OBJECT
{
protected:
    cont touch(LEVEL* level, coor pos, MONSTER*, BUNCH*, OBJECT* object);
    void TouchMonster(LEVEL* level, coor, MONSTER*, BUNCH*, MONSTER* subject);
public:
    RAY_TELEPORT(int value) : (value) { }
};

// @@@ class RAY_LEVEL_TELEPORT

class RAY_LEVEL_TELEPORT : public RAY_OBJECT
{
protected:
    cont touch(LEVEL*, coor, MONSTER*, BUNCH*, OBJECT*);
    void TouchMonster(LEVEL*, coor, MONSTER*, BUNCH*, MONSTER* subject);
public:
    RAY_LEVEL_TELEPORT(int value) : (value) { }
};

// @@@ class RAY_INVISIBLE

class RAY_INVISIBLE : public RAY_OBJECT
{
protected:
    cont touch(LEVEL*, coor, MONSTER*, BUNCH* bunch, OBJECT* object);
public:
    RAY_INVISIBLE(int value) : (value) { }
};

// @@@ class RAY_VISIBLE

class RAY_VISIBLE : public RAY_OBJECT
{
protected:
    cont touch(LEVEL* level, coor, MONSTER*, BUNCH* bunch, OBJECT* object);
public:
    RAY_VISIBLE(int value) : (value) { }
};

// @@@ class RAY_LIGHT

class RAY_LIGHT : public RAY
{
protected:
    void effect(LEVEL* level, MONSTER*, coor pos);
public:
    RAY_LIGHT(int value) : (value) { }
};

// @@@ class RAY_DARK

class RAY_DARK : public RAY
{
protected:
    void effect(LEVEL* level, MONSTER*, coor pos);
public:
    RAY_DARK(int value) : (value) { }
};

// @@ end

#endif RAY_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
