// @ myth - server - weapon.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef WEAPON_HEADER
#define WEAPON_HEADER

// @@ include

#include "server/thing.h"
#include "server/monster.h"

// @@ STONE

class STONE : public WEAPON
{
    // Identification
    //
protected:
    int BaseValue()
    { return 1; }
public:
    object_type type()
    { return WEAPON_STONE; }
    const char* name()
    { return "stone"; }
    long size()
    { return 5L * MyCount; }
    long weight()
    { return 10L * MyCount; }
    int strength()
    { return 15; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 1 + MyValue / 5; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 1 + MyValue / 5; }
    int ThrowBonus(MONSTER*, OBJECT*)
    { return 10; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('s'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new STONE(this, arg); }
    STONE(BINARY* binary) : (binary)
    { create(); }
    STONE(PARSE* parse) : (parse)
    { create(); }
    STONE(STONE* thing, long count) : (thing, count)
    { create(); }
    STONE(long count = 1L) : (count)
    { create(); }
};

// @@ BOW

class BOW : public WEAPON
{
    // Identification
    //
protected:
    int BaseValue()
    { return 100; }
public:
    object_type type()
    { return WEAPON_BOW; }
    const char* name()
    { return "bow"; }
    bool IsBow()
    { return TRUE; }
    int SpecialBonus()
    { return MyValue; }
    long size()
    { return 250L * MyCount; }
    long weight()
    { return 40L * MyCount; }
    int strength()
    { return 5; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('b'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new BOW(this, arg); }
    BOW(BINARY* binary) : (binary)
    { create(); }
    BOW(PARSE* parse) : (parse)
    { create(); }
    BOW(BOW* thing, long count) : (thing, count)
    { create(); }
    BOW(long count = 1L) : (count)
    { create(); }
};

// @@ ARROW

class ARROW : public WEAPON
{
    // Identification
    //
protected:
    int BaseValue()
    { return 10; }
public:
    object_type type()
    { return WEAPON_ARROW; }
    const char* name()
    { return "arrow"; }
    long size()
    { return 25L * MyCount; }
    long weight()
    { return 5L * MyCount; }
    int strength()
    { return 15; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 3; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 6; }
    int ThrowBonus(MONSTER*, OBJECT* weapon)
    { return weapon->IsBow() ? MyValue * weapon->SpecialBonus() : 0; }
    damage_type DamageType()
    { return DAMAGE_STING; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('a'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new ARROW(this, arg); }
    ARROW(BINARY* binary) : (binary)
    { create(); }
    ARROW(PARSE* parse) : (parse)
    { create(); }
    ARROW(ARROW* thing, long count) : (thing, count)
    { create(); }
    ARROW(long count = 1L) : (count)
    { create(); }
};

// @@ DAGGER

class DAGGER : public WEAPON
{
    // Identification
    //
protected:
    int BaseValue()
    { return 25; }
public:
    object_type type()
    { return WEAPON_DAGGER; }
    const char* name()
    { return "dagger"; }
    long size()
    { return 40L * MyCount; }
    long weight()
    { return 100L * MyCount; }
    int strength()
    { return 13; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 2 + MyValue / 2; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 2 + MyValue / 2; }
    damage_type DamageType()
    { return DAMAGE_CUT; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('d'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new DAGGER(this, arg); }
    DAGGER(BINARY* binary) : (binary)
    { create(); }
    DAGGER(PARSE* parse) : (parse)
    { create(); }
    DAGGER(DAGGER* thing, long count) : (thing, count)
    { create(); }
    DAGGER(long count = 1L) : (count)
    { create(); }
};

// @@ MACE

class MACE : public WEAPON
{
    // Identification
    //
protected:
    int BaseValue()
    { return 5; }
public:
    object_type type()
    { return WEAPON_MACE; }
    const char* name()
    { return "mace"; }
    long size()
    { return 300L * MyCount; }
    long weight()
    { return 300L * MyCount; }
    int strength()
    { return 12; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 10 + MyValue / 5; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 10 + MyValue / 5; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('m'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new MACE(this, arg); }
    MACE(BINARY* binary) : (binary)
    { create(); }
    MACE(PARSE* parse) : (parse)
    { create(); }
    MACE(MACE* thing, long count) : (thing, count)
    { create(); }
    MACE(long count = 1L) : (count)
    { create(); }
};

// @@ SWORD

class SWORD : public WEAPON
{
    // Identification
    //
protected:
    int BaseValue()
    { return 250; }
public:
    object_type type()
    { return WEAPON_LONG_SWORD; }
    const char* name()
    { return "sword"; }
    long size()
    { return 175L * MyCount; }
    long weight()
    { return 350L * MyCount; }
    int strength()
    { return 12; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 10 + MyValue; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 25 + MyValue * 2; }
    damage_type DamageType()
    { return DAMAGE_CUT; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('S'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new SWORD(this, arg); }
    SWORD(BINARY* binary) : (binary)
    { create(); }
    SWORD(PARSE* parse) : (parse)
    { create(); }
    SWORD(SWORD* thing, long count) : (thing, count)
    { create(); }
    SWORD(long count = 1L) : (count)
    { create(); }
};

// @@ TWO_HANDED_SWORD

class TWO_HANDED_SWORD : public WEAPON
{
    // Identification
    //
protected:
    int BaseValue()
    { return 350; }
public:
    object_type type()
    { return WEAPON_TWO_HANDED_SWORD; }
    const char* name()
    { return "two-handed sword"; }
    long size()
    { return 300L; }
    long weight()
    { return 500L * MyCount; }
    int strength()
    { return 13 * MyCount; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 10 + MyValue * 3; }
    int HitBonus(MONSTER* attack, MONSTER*)
    { return attack->strength() * 2 - 10; }
    damage_type DamageType()
    { return DAMAGE_CUT; }
    //
    // Administration
    //
private:
    void create()
    { SetLetter('t'); }
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new TWO_HANDED_SWORD(this, arg); }
    TWO_HANDED_SWORD(BINARY* binary) : (binary)
    { create(); }
    TWO_HANDED_SWORD(PARSE* parse) : (parse)
    { create(); }
    TWO_HANDED_SWORD(TWO_HANDED_SWORD* thing, long count) : (thing, count)
    { create(); }
    TWO_HANDED_SWORD(long count = 1L) : (count)
    { create(); }
};

// @@ end

#endif WEAPON_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
